#include <tinyNeoPixel_Static.h>
#include <AbleButtons.h>
#include <EEPROM.h>

#define BUTTON_PIN   PIN_PA5   
#define PIXEL_PIN    PIN_PA6   
#define PIXEL_COUNT  4

byte pixels[PIXEL_COUNT * 3];
using Button = AblePullupClickerButton;
using ButtonList = AblePullupClickerButtonList;

tinyNeoPixel strip = tinyNeoPixel(PIXEL_COUNT, PIXEL_PIN, NEO_GRB, pixels);
Button btn(BUTTON_PIN); 

bool oldState = HIGH;
int showType = 4;
unsigned long lastAnimationTime = 0;
const unsigned long animationInterval = 20;

void setup() {
  if(EEPROM.read(0) == 255){
    EEPROM.write(0, 4);
  }
  showType = EEPROM.read(0);
  btn.begin();

  pinMode(PIXEL_PIN, OUTPUT);

  strip.show();
  strip.setBrightness(50);
}

void loop() {
  btn.handle();
  if(btn.resetClicked()) { // isPressed() is true when the button is pressed.
    showType++;
    if (showType > 10) {
      showType = 0;
    }
    EEPROM.write(0,showType);
  }
  startShow(showType);
}

void startShow(int i) {
  switch (i) {
    case 0: backAndForthChaseRainbow(25);//colorWipe(strip.Color(0, 0, 0), 50);    
      break;
    case 1: colorWipe(strip.Color(255, 0, 0), 50);  
      break;
    case 2: colorWipe(strip.Color(0, 255, 0), 50);  
      break;
    case 3: colorWipe(strip.Color(0, 0, 255), 50);  
      break;
    case 4: theaterChase(strip.Color(127, 127, 127), 50); 
      break;
    case 5: theaterChase(strip.Color(127,   0,   0), 50); 
      break;
    case 6: theaterChase(strip.Color(0,   127, 0), 50);   
      break;
    case 7: rainbow(20);
      break;
    case 8: rainbowCycle(20);
      break;
    case 9: theaterChaseRainbow(50);
      break;
    case 10: rastaMode(100);
      break;
  }
}

void colorWipe(uint32_t c, uint8_t wait) {
  static uint16_t i = 0;
  static unsigned long lastColorChange = 0;

  if (millis() - lastColorChange >= wait) {
    strip.setPixelColor(i, c);
    strip.show();
    i++;
    if (i >= strip.numPixels()) {
      i = 0;
    }
    lastColorChange = millis();
  }
}

void rainbow(uint8_t wait) {
  static uint8_t j = 0;
  static unsigned long lastRainbowChange = 0;

  if (millis() - lastRainbowChange >= wait) {
    uint16_t i;
    for (i = 0; i < strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel((i + j) & 255));
    }
    strip.show();
    j++;
    lastRainbowChange = millis();
  }
}

void rainbowCycle(uint8_t wait) {
  static uint8_t j = 0;
  static unsigned long lastCycleChange = 0;

  if (millis() - lastCycleChange >= wait) {
    uint16_t i;
    for (i = 0; i < strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel(((i * 256 / strip.numPixels()) + j) & 255));
    }
    strip.show();
    j++;
    lastCycleChange = millis();
  }
}

void theaterChase(uint32_t c, uint8_t wait) {
  static int q = 0;
  static unsigned long lastTheaterChange = 0;

  if (millis() - lastTheaterChange >= wait) {
    for (int i = 0; i < strip.numPixels(); i = i + 3) {
      strip.setPixelColor(i + q, c);
    }
    strip.show();
    
    for (int i = 0; i < strip.numPixels(); i = i + 3) {
      strip.setPixelColor(i + q, 0);
    }
    q++;
    if (q >= 3) {
      q = 0;
    }
    lastTheaterChange = millis();
  }
}
void theaterChaseRainbow(uint8_t wait) {
  static uint8_t currentJ = 0;
  static uint8_t currentQ = 0;
  static unsigned long lastChaseRainbowChange = 0;
  if (millis() - lastChaseRainbowChange >= wait) {
    for (int i = 0; i < strip.numPixels(); i = i + 3) {
      strip.setPixelColor(i + currentQ, Wheel((i + currentJ) % 255));
    }
    strip.show();
    for (int i = 0; i < strip.numPixels(); i = i + 3) {
      strip.setPixelColor(i + currentQ, 0);
    }
    currentQ++;
    if(currentQ >=3)
    {
      currentQ = 0;
      currentJ++;
    }
    if (currentJ >= 256) {
      currentJ = 0;
    }
    lastChaseRainbowChange = millis();
  }
}

uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if (WheelPos < 85) {
    return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  }
  if (WheelPos < 170) {
    WheelPos -= 85;
    return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
  WheelPos -= 170;
  return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}
void backAndForthChaseRainbow(uint8_t wait) {
  static int direction = -1; // 1 for forward, -1 for backward
  static int currentPixel = -1;
  static unsigned long lastBackAndForthChange = 0;
  static uint8_t j = 0;

  if (millis() - lastBackAndForthChange >= wait) {
    strip.clear(); // Clear all pixels

    // Set pixels at current position with rainbow color
    strip.setPixelColor(currentPixel, Wheel((j) % 255));
    
    // If moving forward, increment currentPixel, otherwise decrement
    currentPixel += direction;
    
    // If reached the end of strip, change direction
    if (currentPixel == strip.numPixels()) {
      direction = -1;
      currentPixel = strip.numPixels() - 1; // Move back one step
      // delay(100);
    } else if (currentPixel < 0) { // If reached the beginning of strip
      direction = 1;
      currentPixel = 1; // Move forward one step
      // delay(300);
    }

    strip.show();
    lastBackAndForthChange = millis();
    j++;
    
    if (j >= 256) {
      j = 0;
    }
  }
}

void rastaMode(uint8_t wait) {
  static int currentColor = 0; // 0 for green, 1 for yellow, 2 for red
  static unsigned long lastRastaChange = 0;

  if (millis() - lastRastaChange >= wait) {
    int green = 0;
    int yellow = 0;
    int red = 0;

    // Set color intensities based on the current color
    switch (currentColor) {
      case 0: // Green
        green = 255;
        yellow = 0;
        red = 0;
        break;
      case 1: // Yellow
        green = 255;
        yellow = 255;
        red = 0;
        break;
      case 2: // Red
        green = 0;
        yellow = 255;
        red = 255;
        break;
    }

    // Fade the colors gradually
    for (int i = 0; i < strip.numPixels(); i++) {
      strip.setPixelColor(i, strip.Color(green, yellow, red));
    }
    strip.show();

    // Increment the current color
    currentColor = (currentColor + 1) % 3;

    lastRastaChange = millis();
  }
}
